; gpio.s
; Desenvolvido para a placa EK-TM4C1294XL
; Prof. Guilherme de S. Peron	- 12/03/2018
; Prof. Marcos E. P. Monteiro	- 12/03/2018
; Prof. DaLuz           		- 25/02/2022

;################################################################################
	THUMB											; Instrues do tipo Thumb-2
;################################################################################
; Definies dos Registradores Gerais.  Obs: *(EQU)=(EQUATE)*
SYSCTL_RCGCGPIO_R	 		EQU		0x400FE608
SYSCTL_PRGPIO_R		 		EQU		0x400FEA08
; Definies dos Registradores especiais do PORT J
GPIO_PORTJ_AHB_LOCK_R    	EQU		0x40060520
GPIO_PORTJ_AHB_CR_R      	EQU		0x40060524
GPIO_PORTJ_AHB_AMSEL_R   	EQU		0x40060528
GPIO_PORTJ_AHB_PCTL_R    	EQU		0x4006052C
GPIO_PORTJ_AHB_DIR_R     	EQU		0x40060400
GPIO_PORTJ_AHB_AFSEL_R   	EQU		0x40060420
GPIO_PORTJ_AHB_DEN_R     	EQU		0x4006051C
GPIO_PORTJ_AHB_PUR_R     	EQU		0x40060510	
GPIO_PORTJ_AHB_DATA_R    	EQU		0x400603FC
GPIO_PORTJ               	EQU		2_000000100000000	;pg 382 datasheet
; Definies dos Registradores especiais do PORT F
GPIO_PORTF_AHB_LOCK_R    	EQU		0x4005D520
GPIO_PORTF_AHB_CR_R      	EQU		0x4005D524
GPIO_PORTF_AHB_AMSEL_R   	EQU		0x4005D528
GPIO_PORTF_AHB_PCTL_R    	EQU		0x4005D52C
GPIO_PORTF_AHB_DIR_R     	EQU		0x4005D400
GPIO_PORTF_AHB_AFSEL_R   	EQU		0x4005D420
GPIO_PORTF_AHB_DEN_R     	EQU		0x4005D51C
GPIO_PORTF_AHB_PUR_R     	EQU		0x4005D510	
GPIO_PORTF_AHB_DATA_R    	EQU		0x4005D3FC
GPIO_PORTF               	EQU		2_000000000100000	;pg 382 datasheet

;################################################################################
; rea de Cdigo - Tudo abaixo da diretiva a seguir ser armazenado na memria de 
;                  cdigo (ROM/FLASH)
	AREA    |.text|, CODE, READONLY, ALIGN=2
	; Se alguma funo do arquivo for chamada em outro arquivo	
    EXPORT GPIO_Init            ; Permite chamar GPIO_Init de outro arquivo
	EXPORT PortF_Output			; Permite chamar PortN_Output de outro arquivo
	EXPORT PortJ_Input          ; Permite chamar PortJ_Input de outro arquivo
					
;################################################################################
; Funo GPIO_Init
; Parmetro de entrada: No tem
; Parmetro de sada: No tem
;################################################################################
GPIO_Init
; 1. Ativar o clock para a porta setando o bit correspondente no registrador RCGCGPIO,
; aps isso verificar no PRGPIO se a porta est pronta para uso.
; enable clock to GPIOF at clock gating register
	LDR		R0, =SYSCTL_RCGCGPIO_R  		;Carrega o endereo do registrador RCGCGPIO
	MOV		R1, #GPIO_PORTF                 ;Seta o bit da porta F
	ORR		R1, #GPIO_PORTJ					;Seta o bit da porta J, fazendo com OR
	STR		R1, [R0]						;Move para a memria os bits das portas no endereo do RCGCGPIO
    LDR		R0, =SYSCTL_PRGPIO_R			;Carrega o endereo do PRGPIO para esperar os GPIO ficarem prontos
EsperaGPIO
	LDR     R1, [R0]						;L da memria o contedo do endereo do registrador
	MOV     R2, #GPIO_PORTF                 ;Seta os bits correspondentes s portas para fazer a comparao
	ORR     R2, #GPIO_PORTJ                 ;Seta o bit da porta J, fazendo com OR
    TST     R1, R2							;ANDS de R1 com R2
    BEQ     EsperaGPIO					    ;Se o flag Z=1, volta para o lao. Seno continua executando
; 2. Limpar o AMSEL para desabilitar a analgica
	MOV     R1, #0x00						;Colocar 0 no registrador para desabilitar a funo analgica
	LDR     R0, =GPIO_PORTJ_AHB_AMSEL_R     ;Carrega o R0 com o endereo do AMSEL para a porta J
    STR     R1, [R0]						;Guarda no registrador AMSEL da porta J da memria
    LDR     R0, =GPIO_PORTF_AHB_AMSEL_R		;Carrega o R0 com o endereo do AMSEL para a porta F
    STR     R1, [R0]					    ;Guarda no registrador AMSEL da porta F da memria
 ; 3. Limpar PCTL para selecionar o GPIO
    MOV     R1, #0x00					    ;Colocar 0 no registrador para selecionar o modo GPIO
    LDR     R0, =GPIO_PORTJ_AHB_PCTL_R		;Carrega o R0 com o endereo do PCTL para a porta J
    STR     R1, [R0]                        ;Guarda no registrador PCTL da porta J da memria
    LDR     R0, =GPIO_PORTF_AHB_PCTL_R      ;Carrega o R0 com o endereo do PCTL para a porta F
    STR     R1, [R0]                        ;Guarda no registrador PCTL da porta F da memria
; 4. DIR para 0 se for entrada, 1 se for sada
    LDR     R0, =GPIO_PORTF_AHB_DIR_R		;Carrega o R0 com o endereo do DIR para a porta F
	MOV     R1, #2_00010001					;PF4 & PF0 para LED
    STR     R1, [R0]						;Guarda no registrador
			; O certo era verificar os outros bits da PF para no transformar entradas em sadas desnecessrias
    LDR     R0, =GPIO_PORTJ_AHB_DIR_R		;Carrega o R0 com o endereo do DIR para a porta J
    MOV     R1, #0x00               		;Colocar 0 no registrador DIR para funcionar com sada
    STR     R1, [R0]						;Guarda no registrador PCTL da porta J da memria
; 5. Limpar os bits AFSEL para 0 para selecionar GPIO, Sem funo alternativa
    MOV     R1, #0x00						;Colocar o valor 0 para no setar funo alternativa
    LDR     R0, =GPIO_PORTF_AHB_AFSEL_R		;Carrega o endereo do AFSEL da porta F
    STR     R1, [R0]						;Escreve na porta
    LDR     R0, =GPIO_PORTJ_AHB_AFSEL_R     ;Carrega o endereo do AFSEL da porta J
    STR     R1, [R0]                        ;Escreve na porta
; 6. Setar os bits de DEN para habilitar I/O digital
    LDR     R0, =GPIO_PORTF_AHB_DEN_R		;Carrega o endereo do DEN
    MOV     R1, #2_00010001                 ;Ativa os pinos PF0 e PF4 como I/O Digital
    STR     R1, [R0]						;Escreve no registrador da memria funcionalidade digital 
    LDR     R0, =GPIO_PORTJ_AHB_DEN_R		;Carrega o endereo do DEN
	MOV     R1, #2_00000011                 ;Ativa os pinos PJ0 e PJ1 como I/O Digital      
    STR     R1, [R0]                        ;Escreve no registrador da memria funcionalidade digital
; 7. Para habilitar resistor de pull-up interno, setar PUR para 1
	LDR     R0, =GPIO_PORTJ_AHB_PUR_R		;Carrega o endereo do PUR para a porta J
	MOV     R1, #2_00000011					;Habilitar funcionalidade digital de resistor de pull-up nos bits 0 e 1
    STR     R1, [R0]						;Escreve no registrador da memria do resistor de pull-up
;retorno            
	BX      LR								;Retorna da Chamada da Funo

;################################################################################
; Funo PortF_Output
; Parmetro de entrada: R0 ==> se os BIT4 e BIT0 esto ligado ou desligado
; Parmetro de sada: No tem
;################################################################################
PortF_Output
	LDR		R1, =GPIO_PORTF_AHB_DATA_R		;Carrega o valor do offset do data register
											;Read-Modify-Write para escrita
	LDR 	R2, [R1]						;Carrega o valor do PORTF(leitura) em R2;
	BIC 	R2, #2_00010001                 ;Primeiro limpamos os dois bits do lido da porta R2 = R2 & 11101110
	ORR 	R0, R0, R2                      ;Fazer o OR do lido pela porta com o parmetro de entrada
	STR 	R0, [R1]                        ;Escreve na porta F o barramento de dados dos pinos F4 e F0
	BX 		LR								;Retorna da Chamada da Funo

;################################################################################
; Funo PortJ_Input
; Parmetro de entrada: No tem
; Parmetro de sada: R0 ==> o valor da leitura
;################################################################################
PortJ_Input
	LDR		R1, =GPIO_PORTJ_AHB_DATA_R	    ;Carrega o valor do offset do data register
	LDR 	R0, [R1]                        ;L no barramento de dados dos pinos [J1-J0]
	BX 		LR								;Retorno

    ALIGN                           		; garante que o fim da seo est alinhada 
    END                             		; fim do arquivo